<?php

namespace App\Http\Controllers\Api;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Http\Request;
use App\Models\User;

class LeaderController extends Controller
{
    public function LeaderLogin(Request $request){

        $validator = Validator::make($request->all(), [
            'email_or_phone' => [
                'required',
                function ($attribute, $value, $fail) {
                    if (!filter_var($value, FILTER_VALIDATE_EMAIL) && !preg_match('/^\d{10,12}$/', $value)) {
                        $fail('The email or phone number must be a valid email or a phone number with 10-12 digits.');
                    }
                }
            ],
            'password' => 'required',
        ]);
    
        if ($validator->fails()) {
            return response()->json([
                'status' => 422,
                'validation_errors' => $validator->errors(),
                'message' => 'Validation failed',
            ], 422);
        }

        $validatedData = $validator->validated();
        
        $user = User::where('email', $validatedData['email_or_phone'] ?? null)
                ->orWhere('phone_no', $validatedData['email_or_phone'] ?? null)
                ->first();
        
        // Check if user exists, password matches, and 
        if (!$user || !Hash::check($validatedData['password'], $user->password)) {
            return response()->json([
                'status' => 401,
                'message' => 'Invalid email ,phone_no or password',
            ], 401); 
        }

        if (!in_array($user->user_role, ['Pastor', 'Prayer (Leader)', 'Senior Prayer Warrior (Leader)'])) {
            return response()->json([
                'status' => 403,
                'message' => 'Unauthorized access. Only users with the Leader role are authorized.',
            ], 403);
        }
        
        if ($user->user_status == 'Inactive User') {
            return response()->json([
                'status' => 403,
                'message' => 'Your Account is Inactive, please contact to administrator.',
            ], 403);
        }
            
        // Generate authentication token
         $token = $user->createToken($user->email . 'Token')->plainTextToken;
         $church_name = $user->church ? $user->church->church_name : null;
         $church_logo = $user->church ? asset($user->church->church_logo) : null;


        return response()->json([
            'status' => 200,
            'message' => 'Login successful',
            'user' => [
                'first_name' => $user->first_name,
                'last_name' => $user->last_name,
                'phone_no' => $user->phone_no,
                'email'=> $user->email,
                'user_role'=> $user->user_role,
                'user_status'=> $user->user_status,
                'church_name' => $church_name,
                'profile_photo' => asset($user->profile_photo),
                'church_logo' => $church_logo,
            ],
            
            'token' => $token,
        ], 200);
    }

    public function ViewProfile(Request $request)
    {
        try {
            $user = Auth::user();
    
            if (!$user) {
                return response()->json([
                    'status' => 401,
                    'message' => 'Unauthorized: User not authenticated',
                ], 401);
            }
    
            return response()->json([
                'status' => 200,
                'message' => 'Profile retrieved successfully',
                'user' => [
                    'first_name' => $user->first_name,
                    'last_name' => $user->last_name,
                    'phone_no' => $user->phone_no,
                    'email' => $user->email,
                    'user_role' => $user->user_role,
                    'user_status' => $user->user_status,
                    'profile_photo' =>  $user->profile_photo 
                    ? asset($user->profile_photo) 
                    : asset('user_profile/profile.png')
                ],
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => 500,
                'message' => 'An error occurred while retrieving the profile',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    
    public function UpdateProfile(Request $request)
    {
        try {
            $user = Auth::user();
    
            if (!$user) {
                return response()->json([
                    'status' => 404,
                    'message' => 'User not found',
                ], 404);
            }
    
            // Validate request data
            $validator = Validator::make($request->all(), [
                'first_name' => 'required|string|max:255',
                'last_name' => 'required|string|max:255',
                'phone_no' => 'required|digits:10|unique:users,phone_no,' . $user->id,
     
            ]);
    
            if ($validator->fails()) {
                return response()->json([
                    'status' => 422,
                    'validation_errors' => $validator->errors(),
                    'message' => 'Validation failed',
                ], 422);
            }
    
            // Update user profile using validated data
            $user->update($validator->validated());

                     // Update user profile
                $user->first_name = $request->input('first_name');
                $user->last_name = $request->input('last_name');
                $user->phone_no = $request->input('phone_no');
                $user->save();
         
    
            return response()->json([
                'status' => 200,
                'message' => 'Profile updated successfully',
                'user' => [
                    'first_name' => $user->first_name,
                    'last_name' => $user->last_name,
                    'phone_no' => $user->phone_no,
                    'email' => $user->email,
                    'user_role' => $user->user_role,
                    'user_status' => $user->user_status,
                    'profile_photo' => $user->profile_photo 
                    ? asset($user->profile_photo) 
                    : asset('user_profile/profile.png')
                ],
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => 500,
                'message' => 'An error occurred while updating the profile',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function UpdateProfilePic(Request $request)
    {
       
        $user = Auth::user();
    
     
        if (!$user) {
            return response()->json([
                'status' => 401,
                'message' => 'Unauthorized: User not authenticated',
            ], 401);
        }
    
        $validator = Validator::make($request->all(), [
            'profile_photo' => 'required|image|mimes:jpeg,png,jpg,gif|max:2048', 
        ]);
    
        if ($validator->fails()) {
            return response()->json([
                'status' => 422,
                'message' => 'Validation failed',
                'errors' => $validator->errors(),
            ], 422);
        }

        if ($request->hasFile('profile_photo')) {
  
            $file = $request->file('profile_photo');
            $filename = 'profile_' . uniqid() . '.' . $file->getClientOriginalExtension(); 
    

            $path = public_path('images/user_profile/' . $filename);
            $file->move(public_path('images/user_profile'), $filename); 
    
        
            $user->profile_photo = 'images/user_profile/' . $filename; 
            $user->save();
    
            return response()->json([
                'status' => 200,
                'message' => 'Profile photo updated successfully',
                'profile_photo' => asset('images/user_profile/' . $filename), 
            ], 200);
        }
  
        return response()->json([
            'status' => 400,
            'message' => 'No file uploaded',
        ], 400);
    }
    
}
