<?php

namespace App\Http\Controllers\Api;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\Church;

class UserController extends Controller
{

    public function UserLogin(Request $request){

        $validator = Validator::make($request->all(), [
            'email_or_phone' => [
                'required',
                function ($attribute, $value, $fail) {
                    if (!filter_var($value, FILTER_VALIDATE_EMAIL) && !preg_match('/^\d{10,12}$/', $value)) {
                        $fail('The email or phone number must be a valid email or a phone number with 10-12 digits.');
                    }
                }
            ],
            'password' => 'required',
        ]);
    
        if ($validator->fails()) {
            return response()->json([
                'status' => 422,
                'validation_errors' => $validator->errors(),
                'message' => 'Validation failed',
            ], 422);
        }

        $validatedData = $validator->validated();

        
        $user = User::where('email', $validatedData['email_or_phone'] ?? null)
                ->orWhere('phone_no', $validatedData['email_or_phone'] ?? null)
                ->first();
        
        // Check if user exists, password matches, and 
        if (!$user || !Hash::check($validatedData['password'], $user->password)) {
            return response()->json([
                'status' => 401,
                'message' => 'Invalid email ,phone_no or password',
            ], 401); 
        }

        
        if ($user->user_role !== 'Prayer Warrior') {
            return response()->json([
                'status' => 403,
                'message' => 'Unauthorized access. Only users with the Prayer Warrior role are authorized.',
            ], 403);
        }
        
        if ($user->user_status == 'Inactive User') {
            return response()->json([
                'status' => 403,
                'message' => 'Your Account is Inactive, please contact to administrator.',
            ], 403);
        }
            
        // Generate authentication token
         $token = $user->createToken($user->email . 'Token')->plainTextToken;
         $church_name = $user->church ? $user->church->church_name : null;
         $church_logo = $user->church ? asset($user->church->church_logo) : null;


            return response()->json([
                'status' => 200,
                'message' => 'Login successful',
                'user' => [
                    'first_name' => $user->first_name,
                    'last_name' => $user->last_name,
                    'phone_no' => $user->phone_no,
                    'email'=> $user->email,
                    'user_role'=> $user->user_role,
                    'user_status'=> $user->user_status,
                    'church_name' => $church_name,
                    'profile_photo' => asset($user->profile_photo),
                    'church_logo' => $church_logo,
                ],
             
                'token' => $token,
            ], 200);
    }

    public function ViewProfile(Request $request)
    {
        try {
            $user = Auth::user();
    
            if (!$user) {
                return response()->json([
                    'status' => 401,
                    'message' => 'Unauthorized: User not authenticated',
                ], 401);
            }
    
            return response()->json([
                'status' => 200,
                'message' => 'Profile retrieved successfully',
                'user' => [
                    'first_name' => $user->first_name,
                    'last_name' => $user->last_name,
                    'phone_no' => $user->phone_no,
                    'email' => $user->email,
                    'user_role' => $user->user_role,
                    'user_status' => $user->user_status,
                    'profile_photo' =>  $user->profile_photo 
                    ? asset($user->profile_photo) 
                    : asset('user_profile/profile.png')
                ],
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => 500,
                'message' => 'An error occurred while retrieving the profile',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    
    public function UpdateProfile(Request $request)
    {
        try {
            $user = Auth::user();
    
            if (!$user) {
                return response()->json([
                    'status' => 404,
                    'message' => 'User not found',
                ], 404);
            }
    
            // Validate request data
            $validator = Validator::make($request->all(), [
                'first_name' => 'required|string|max:255',
                'last_name' => 'required|string|max:255',
                'phone_no' => 'required|digits:10|unique:users,phone_no,' . $user->id,
            ]);
    
            if ($validator->fails()) {
                return response()->json([
                    'status' => 422,
                    'validation_errors' => $validator->errors(),
                    'message' => 'Validation failed',
                ], 422);
            }
    
            // Update user profile using validated data
            $user->update($validator->validated());

                     // Update user profile
                $user->first_name = $request->input('first_name');
                $user->last_name = $request->input('last_name');
                $user->phone_no = $request->input('phone_no');
                $user->save();
         
    
            return response()->json([
                'status' => 200,
                'message' => 'Profile updated successfully',
                'user' => [
                    'first_name' => $user->first_name,
                    'last_name' => $user->last_name,
                    'phone_no' => $user->phone_no,
                    'email' => $user->email,
                    'user_role' => $user->user_role,
                    'user_status' => $user->user_status,
                    'profile_photo' => $user->profile_photo 
                    ? asset($user->profile_photo) 
                    : asset('user_profile/profile.png')
                ],
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => 500,
                'message' => 'An error occurred while updating the profile',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    
    public function AddUser(Request $request)
    {
        try {
            // Validate request data
            $validator = Validator::make($request->all(), [
                'first_name' => 'required|string|max:255',
                'last_name' => 'required|string|max:255',
                'phone_no' => 'required|digits:10|unique:users,phone_no',
                'email' => 'required|email|unique:users,email',
                'user_role' => 'required|string',
                'user_status' => 'required|string',
                'password' => 'nullable|string|min:6', 
            ]);
    
            if ($validator->fails()) {
                return response()->json([
                    'status' => 422,
                    'validation_errors' => $validator->errors(),
                    'message' => 'Validation failed',
                ], 422);
            }

            $hashedPassword = Hash::make($request->input('password', 'user@123'));
            $user = Auth::user(); 
            $church_id = $user->church_id;
            $addedBy = $user->id;
         
            // Create user with default password if not provided
            $user = User::create([
                'first_name' => $request->input('first_name'),
                'last_name' => $request->input('last_name'),
                'phone_no' => $request->input('phone_no'),
                'email' => $request->input('email'),
                'user_role' => $request->input('user_role'),
                'user_status' => $request->input('user_status'),
                'church_id'=>$church_id,
                'added_by'=>$addedBy,
                'password' => $hashedPassword, 
            ]);
    
            return response()->json([
                'status' => 201,
                'message' => 'User added successfully',
                'user' => $user,
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => 500,
                'message' => 'An error occurred while adding the user',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function ShowUser(){

        try {
            $loggedInUser = Auth::user();
    
            $users = User::where('id', '!=', $loggedInUser->id)->get();
    
            $formattedUsers = $users->map(function ($user) {
                $profilePhoto = $user->profile_photo 
                ? asset($user->profile_photo) 
                : asset('user_profile/profile.png');

                return [
                    'id'=>$user->id,
                    'first_name' => $user->first_name,
                    'last_name' => $user->last_name,
                    'phone_no' => $user->phone_no,
                    'email' => $user->email,
                    'user_role' => $user->user_role,
                    'user_status' => $user->user_status,
                    'profile_photo' => $profilePhoto,
                ];
            });
    
            return response()->json([
                'status' => 200,
                'message' => 'Users retrieved successfully',
                'users' => $formattedUsers, // Return the formatted users
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => 500,
                'message' => 'An error occurred while retrieving users',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    
    public function EditUser(Request $request, $id){

        try {
            $user = User::find($id);
            if (!$user) {
                return response()->json([
                    'status' => 404,
                    'message' => 'User not found',
                ], 404);
            }
            return response()->json([
                'status' => 200,
                'message' => 'User retrieved successfully',
                'user' => [
                    'id'=>$user->id,
                    'first_name' => $user->first_name,
                    'last_name' => $user->last_name,
                    'phone_no' => $user->phone_no,
                    'email' => $user->email,
                    'user_role' => $user->user_role,
                    'user_status' => $user->user_status,
                    'profile_photo' =>  $user->profile_photo 
                    ? asset($user->profile_photo) 
                    : asset('user_profile/profile.png')
                ],

            ], 200);
        } catch (\Exception $e) {
           
            return response()->json([
                'status' => 500,
                'message' => 'An error occurred while retrieving the user',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    
    public function UpdateUser(Request $request, $id)
    {
        try {
 
               $user = User::find($id);
               if (!$user) {
                return response()->json([
                    'status' => 404,
                    'message' => 'User not found',
                ], 404);
            }

            // Validate incoming data
            $validator = Validator::make($request->all(), [
                'first_name' => 'required|string|max:255',
                'last_name' => 'required|string|max:255',
                'phone_no' => 'required|digits:10|unique:users,phone_no,' . $id,
                'email' => 'required|email|unique:users,email,' . $id,
                'user_role' => 'required|string',
                'user_status' => 'required|string',
            ]);
    
            if ($validator->fails()) {
                return response()->json([
                    'status' => 422,
                    'validation_errors' => $validator->errors(),
                    'message' => 'Validation failed',
                ], 422);
            }
    
         
            $user->first_name = $request->input('first_name');
            $user->last_name = $request->input('last_name');
            $user->phone_no = $request->input('phone_no');
            $user->email = $request->input('email');
            $user->user_role = $request->input('user_role');
            $user->user_status = $request->input('user_status');
            $user->save();
    
            return response()->json([
                'status' => 200,
                'message' => 'User updated successfully',
               'user' => [
                    'id'=>$user->id,
                    'first_name' => $user->first_name,
                    'last_name' => $user->last_name,
                    'phone_no' => $user->phone_no,
                    'email' => $user->email,
                    'user_role' => $user->user_role,
                    'user_status' => $user->user_status,
                ],
            ], 200);
    
        } catch (\Exception $e) {
            return response()->json([
                'status' => 500,
                'message' => 'An error occurred while updating the user',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function DeleteUser($id){
        try {
            $user = User::find($id);
    
            if (!$user) {
                return response()->json([
                    'status' => 404,
                    'message' => 'User not found',
                ], 404);
            }
            $user->delete();
    
            return response()->json([
                'status' => 200,
                'message' => 'User deleted successfully',
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => 500,
                'message' => 'An error occurred while deleting the user',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    
   
    public function UpdateProfilePic(Request $request)
    {
       
        $user = Auth::user();
     
        if (!$user) {
            return response()->json([
                'status' => 401,
                'message' => 'Unauthorized: User not authenticated',
            ], 401);
        }
    
        $validator = Validator::make($request->all(), [
            'profile_photo' => 'required|image|mimes:jpeg,png,jpg,gif|max:2048', 
        ]);
    
        if ($validator->fails()) {
            return response()->json([
                'status' => 422,
                'message' => 'Validation failed',
                'errors' => $validator->errors(),
            ], 422);
        }

        if ($request->hasFile('profile_photo')) {
  
            $file = $request->file('profile_photo');
            $filename = 'profile_' . uniqid() . '.' . $file->getClientOriginalExtension(); 
    

            $path = public_path('images/user_profile/' . $filename);
            $file->move(public_path('images/user_profile'), $filename); 
    
        
            $user->profile_photo = 'images/user_profile/' . $filename; 
            $user->save();
    
            return response()->json([
                'status' => 200,
                'message' => 'Profile photo updated successfully',
                'profile_photo' => asset('images/user_profile/' . $filename), 
            ], 200);
        }
  
        return response()->json([
            'status' => 400,
            'message' => 'No file uploaded',
        ], 400);
    }
}
